pub struct MonoWaitGroup(/* private fields */);Expand description
WaitGroup with a single non-clonable worker handle.
Cancellation safety
This future is cancellation safe.
It is also safe to poll again after completion.
let (wg, handle) = MonoWaitGroup::new();
let mut wg = core::pin::pin!(wg);
assert!(!wg.is_done());
handle.done();
wg.as_mut().await;
assert!(wg.is_done());
// It is safe to await again (re-poll)
wg.as_mut().await;
assert!(wg.is_done());Implementations
Sourceimpl MonoWaitGroup
impl MonoWaitGroup
Sourcepub fn new() -> (Self, MonoWorkerHandle)
pub fn new() -> (Self, MonoWorkerHandle)
Creates a new MonoWaitGroup and a single MonoWorkerHandle.
This variant is optimized for scenarios where there is exactly one worker task. The handle cannot be cloned.
Examples
use compact_waitgroup::MonoWaitGroup;
let (wg, handle) = MonoWaitGroup::new();Trait Implementations
Sourceimpl Debug for MonoWaitGroup
impl Debug for MonoWaitGroup
Sourceimpl Future for MonoWaitGroup
impl Future for MonoWaitGroup