pub struct WaitGroup(/* private fields */);Expand description
WaitGroup with clonable worker handles.
Cancellation safety
This future is cancellation safe.
It is also safe to poll again after completion.
let (wg, handle) = WaitGroup::new();
let mut wg = core::pin::pin!(wg);
assert!(!wg.is_done());
handle.done();
wg.as_mut().await;
assert!(wg.is_done());
// It is safe to await again (re-poll)
wg.as_mut().await;
assert!(wg.is_done());Implementations
Sourceimpl WaitGroup
impl WaitGroup
Sourcepub fn new() -> (Self, WorkerHandle)
pub fn new() -> (Self, WorkerHandle)
Creates a new WaitGroup and a clonable WorkerHandle.
The WaitGroup is used to await the completion of tasks. The
WorkerHandle is used to signal task completion.
Examples
use compact_waitgroup::WaitGroup;
let (wg, handle) = WaitGroup::new();
// ... distribute handle ...Trait Implementations