pub struct MonoWaitGroup(/* private fields */);Expand description
WaitGroup with a single non-clonable group token.
This variant is optimized for scenarios where there is exactly one worker task, and the group token cannot be cloned.
Cancellation safety
This future is cancellation safe.
It is also safe to poll again after completion.
let (wg, token) = MonoWaitGroup::new();
let mut wg = core::pin::pin!(wg);
assert!(!wg.is_done());
token.release();
wg.as_mut().await;
assert!(wg.is_done());
// It is safe to await again (re-poll)
wg.as_mut().await;
assert!(wg.is_done());Implementations
Sourceimpl MonoWaitGroup
impl MonoWaitGroup
Sourcepub fn new() -> (Self, MonoGroupToken)
pub fn new() -> (Self, MonoGroupToken)
Creates a new MonoWaitGroup and a single MonoGroupToken.
Sourcepub fn is_done(&self) -> bool
pub fn is_done(&self) -> bool
Checks if the MonoWaitGroup has completed.
This returns true if the MonoGroupToken has been dropped.