MonoWaitGroup

Struct MonoWaitGroup 

pub struct MonoWaitGroup(/* private fields */);
WaitGroup with a single non-clonable worker handle.

Cancellation safety

This future is cancellation safe.

It is also safe to poll again after completion.

let (wg, handle) = MonoWaitGroup::new();
let mut wg = core::pin::pin!(wg);

assert!(!wg.is_done());

handle.done();

wg.as_mut().await;
assert!(wg.is_done());

// It is safe to await again (re-poll)
wg.as_mut().await;
assert!(wg.is_done());

Implementations

Source

impl MonoWaitGroup

pub fn new() -> (Self, MonoWorkerHandle)

Creates a new MonoWaitGroup and a single MonoWorkerHandle.

This variant is optimized for scenarios where there is exactly one worker task. The handle cannot be cloned.

Examples
use compact_waitgroup::MonoWaitGroup;

let (wg, handle) = MonoWaitGroup::new();
pub fn is_done(&self) -> bool

Checks if the MonoWaitGroup has completed.

This returns true if the MonoWorkerHandle has been dropped.

Examples
use compact_waitgroup::MonoWaitGroup;

let (wg, handle) = MonoWaitGroup::new();
assert!(!wg.is_done());

drop(handle);
assert!(wg.is_done());